/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2018 Tibor 'Igor2' Palinkas
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.*
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */

#include "config.h"

#include <stdio.h>
#include <stdarg.h>
#include <genvector/vtp0.h>
#include <librnd/core/fptr_cast.h>

#include "message.h"

vtp0_t handlers;
csch_message_level_t min_level = CSCH_MSG_INFO;

void csch_message(csch_message_level_t level, const char *format, ...)
{
	va_list args;
	int handled = 0, n;

	if (level < min_level)
		return;

	for(n = 0; n < vtp0_len(&handlers); n++) {
		csch_message_handler_t fp = (csch_message_handler_t)rnd_cast_d2f(handlers.array[n]);
		va_start(args, format);
		handled |= fp(level, format, args);
		va_end(args);
	}

 /* fallback */
	if (!handled) {
		va_start(args, format);
		vfprintf(stderr, format, args);
		va_end(args);
	}
}

/* The lack of va_copy in c89 forces us to copy the whole function */
void csch_error(const char *format, ...)
{
	va_list args;
	int handled = 0, n;

	if (CSCH_MSG_ERROR < min_level)
		return;

	for(n = 0; n < vtp0_len(&handlers); n++) {
		csch_message_handler_t fp = (csch_message_handler_t)rnd_cast_d2f(handlers.array[n]);
		va_start(args, format);
		handled |= fp(CSCH_MSG_ERROR, format, args);
		va_end(args);
	}

 /* fallback */
	if (!handled) {
		va_start(args, format);
		vfprintf(stderr, format, args);
		va_end(args);
	}
}

static int find_handler(csch_message_handler_t handler)
{
	int n;
	void *f = rnd_cast_f2d((rnd_fptr_t)handler);
	for(n = 0; n < vtp0_len(&handlers); n++)
		if (handlers.array[n] == f)
			return n;
	return -1;
}

void csch_message_reg_handler(csch_message_handler_t handler)
{
	void *f = rnd_cast_f2d((rnd_fptr_t)handler);

	if (find_handler(handler) >= 0)
		return;

	vtp0_append(&handlers, f);
}

void csch_message_unreg_handler(csch_message_handler_t handler)
{
	int idx = find_handler(handler);
	if (idx > 0)
		vtp0_remove(&handlers, idx, 1);
}

